/**************************************************************************//**
 * @file     main.c
 * @version  V3.00
 * $Revision: 5 $
 * $Date: 15/09/02 10:04a $
 * @brief    Demonstrate measure internal temperature sensor voltage.
 * @note
 * Copyright (C) 2019 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include "stdio.h"
#include "M451Series.h"

#define PLLCTL_SETTING      CLK_PLLCTL_72MHz_HXT
#define PLL_CLOCK           72000000

/*---------------------------------------------------------------------------------------------------------*/
/* Define global variables and constants                                                                   */
/*---------------------------------------------------------------------------------------------------------*/
#define ADC_AVERAGE_TIMES   10

static volatile uint8_t u8ConvertTimes;
static volatile uint8_t u8ADCStop;
static volatile uint32_t u32ConvertResVgap,u32ConvertResVtemp;
static volatile uint32_t u32AverageValueVgap,u32AverageValueVtemp;
static volatile uint32_t Vgap,Vtemp;
static volatile float fVDD,Check;
/*---------------------------------------------------------------------------------------------------------*/
/* Define functions prototype                                                                              */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void);
void EADC_FunctionTest(void);


void SYS_Init(void)
{

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable HIRC clock (Internal RC 22.1184MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRCEN_Msk);

    /* Wait for HIRC clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* Select HCLK clock source as HIRC and and HCLK source divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV0_HCLK(1));

    /* Set PLL to Power-down mode and PLLSTB bit in CLK_STATUS register will be cleared by hardware.*/
    CLK_DisablePLL();

    /* Enable HXT clock (external XTAL 12MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Wait for HXT clock ready */
    CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK);

    /* Enable UART module clock */
    CLK_EnableModuleClock(UART0_MODULE);

    /* Select UART module clock source as HXT and UART module clock divider as 1 */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UARTSEL_HXT, CLK_CLKDIV0_UART(1));

    /* Enable EADC module clock */
    CLK_EnableModuleClock(EADC_MODULE);

    /* EADC clock source is 72MHz, set divider to 8, ADC clock is 72/8 MHz */
    CLK_SetModuleClock(EADC_MODULE, 0, CLK_CLKDIV0_EADC(8));

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Set PD multi-function pins for UART0 RXD and TXD */
    SYS->GPD_MFPL &= ~(SYS_GPD_MFPL_PD0MFP_Msk | SYS_GPD_MFPL_PD1MFP_Msk);
    SYS->GPD_MFPL |= (SYS_GPD_MFPL_PD0MFP_UART0_RXD | SYS_GPD_MFPL_PD1MFP_UART0_TXD);

    /* Disable the GPB0 - GPB3 digital input path to avoid the leakage current. */
    GPIO_DISABLE_DIGITAL_PATH(PB, 0xF);

}

void UART0_Init()
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART module */
    SYS_ResetModule(UART0_RST);

    /* Configure UART0 and set UART0 baud rate */
    UART_Open(UART0, 115200);
}

/*---------------------------------------------------------------------------------------------------------*/
/* EADC function test                                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
void EADC_FunctionTest()
{



    /* Enable channel 16 and select internal band-gap voltage as input source */
    EADC_Open(EADC, EADC_CTL_DIFFEN_SINGLE_END);
    /* Set sample module 16 & 17 external sampling time to 0xF */
    EADC_SetExtendSampleTime(EADC, 16, 0xF);
    EADC_SetExtendSampleTime(EADC, 17, 0xF);

    EADC_ConfigSampleModule(EADC, 16, EADC_SOFTWARE_TRIGGER, 0);
    EADC_ConfigSampleModule(EADC, 17, EADC_SOFTWARE_TRIGGER, 0);
    SYS->IVSCTL |= SYS_IVSCTL_VTEMPEN_Msk;
    /* Clear the A/D ADINT0 interrupt flag for safe */
    EADC_CLR_INT_FLAG(EADC, EADC_STATUS2_ADIF0_Msk);
    /* Enable the sample module 17 interrupt.  */
    EADC_ENABLE_INT(EADC, BIT0);
    EADC_ENABLE_SAMPLE_MODULE_INT(EADC, 0, BIT17);
    NVIC_EnableIRQ(ADC00_IRQn);

    u8ADCStop = 0;
    u8ConvertTimes = 0;
    u32ConvertResVgap = 0;
    u32ConvertResVtemp = 0;

		/* Trigger ADC conversion */
    while(1) {
        if(!(EADC_IS_BUSY(EADC))) {
            EADC_START_CONV(EADC, BIT16|BIT17);
        }

        if(u8ADCStop) break;
    }

    /* Disable the ADINT0 interrupt */
    EADC_DISABLE_INT(EADC, BIT0);
    NVIC_DisableIRQ(ADC00_IRQn);

    /* Calculate average value of ADC result */
    u32AverageValueVgap = u32ConvertResVgap/ADC_AVERAGE_TIMES;
    u32AverageValueVtemp = u32ConvertResVtemp/ADC_AVERAGE_TIMES;
    
    /* The equation of converting to real temperature is as below 
     * (748-(i32ConversionData*ADCVREF/4095))/(1.672)), ADCVREF means the VREF voltage get from Band-gap 1.2V
     */
    fVDD=(1.2f*4095*1000/u32AverageValueVgap);
    Check=(double)(748-(u32AverageValueVtemp*fVDD/4095))/1.672;

    printf("Temperature: %f\n",Check);
}

/*---------------------------------------------------------------------------------------------------------*/
/* EADC interrupt handler                                                                                  */
/*---------------------------------------------------------------------------------------------------------*/
void ADC00_IRQHandler(void)
{

    
     /* Clear ADC convert complete flag */
    EADC_CLR_INT_FLAG(EADC, EADC_STATUS2_ADIF0_Msk);
    Vgap = EADC_GET_CONV_DATA(EADC, 16);
    Vtemp = EADC_GET_CONV_DATA(EADC, 17);
    
    /* Get ADC convert result */
    if(u8ConvertTimes ++ < ADC_AVERAGE_TIMES)
    {
        u32ConvertResVgap += Vgap;
        u32ConvertResVtemp += Vtemp;
    }
    else if(u8ConvertTimes >= ADC_AVERAGE_TIMES)
        u8ADCStop = 1;
}

/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void)
{

    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Init System, IP clock and multi-function I/O */
    SYS_Init();

    /* Lock protected registers */
    SYS_LockReg();

    /* Init UART0 for printf */
    UART0_Init();

    /*---------------------------------------------------------------------------------------------------------*/
    /* SAMPLE CODE                                                                                             */
    /*---------------------------------------------------------------------------------------------------------*/

    printf("\nSystem clock rate: %d Hz\n", SystemCoreClock);

    /* EADC function test */
    EADC_FunctionTest();

    /* Reset EADC module */
    SYS_ResetModule(EADC_RST);

    /* Disable EADC IP clock */
    CLK_DisableModuleClock(EADC_MODULE);

    /* Disable External Interrupt */
    NVIC_DisableIRQ(ADC00_IRQn);

    printf("Exit EADC sample code\n");

    while(1);

}

/*** (C) COPYRIGHT 2019 Nuvoton Technology Corp. ***/
